package jpmapper.core;


import java.lang.reflect.InvocationTargetException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;

import jpmapper.config.DBInformation;
import jpmapper.exceptions.JPMapperException;
import jpmapper.exceptions.LockWaitTimeoutException;


/**
 * @author dahhane walid, mindware.ma
 * @version 0.7.1
 */
public abstract class AbstractDBMapper {
	
	protected DBInformation dbInformation;
	protected JPMapperException ormException;
	protected JPMapperFacade facadeORM;
	//association is the mapping of DataBase Type and Java Type for example: varchar to String
	protected HashMap<String, Class> dbType2JavaType;
	protected HashMap<String, String> dbTypeWithDelimiter;
	protected HashMap<Class, String> javaTypeWithDelimiter;
	protected Connection connection = null;

	public AbstractDBMapper(){

	}

	public void finalize() throws Throwable {

	}

	public abstract void commit();

	/**
	 * 
	 * @throws JPMapperException
	 */
	public abstract void connect(String host,int port, String dbName,String user, String password,boolean isAutoCommit) throws JPMapperException;

	/**
	 * 
	 * @param object
	 * @param where
	 * @throws LockWaitTimeoutException 
	 * @throws SQLException 
	 */
	public abstract int delete(Object object, String where) throws LockWaitTimeoutException, SQLException;

	
	/**
	 * 
	 * @throws JPMapperException
	 */
	public abstract void disconnect();

	/**
	 * 
	 * @param query
	 * @throws JPMapperException
	 */
	public abstract boolean execute(String query) throws JPMapperException;

	/**
	 * 
	 * @param query
	 * @throws JPMapperException
	 */
	public abstract int executeUpdate(String query) throws JPMapperException;
	
	/**
	 * 
	 * @param query
	 * @throws JPMapperException
	 */
	public abstract ResultSet executeQuery(String query) throws JPMapperException;

	/**
	 * 
	 * @param object
	 * @throws JPMapperException
	 */
	public abstract long getNextAutoIncrement(Object object) throws JPMapperException;

	/**
	 * 
	 * @param object
	 * @throws JPMapperException
	 */
	public abstract boolean insert(Object object) throws JPMapperException;
	public abstract Object map(Object parentObject,Object object);
	public abstract ArrayList mapAll(Object parentObject,Object arrayList) throws JPMapperException, IllegalArgumentException, SecurityException, IllegalAccessException, InvocationTargetException, NoSuchMethodException;

	public abstract ArrayList mapAll(ResultSet rs,Object object) throws JPMapperException;
	
	public abstract Object map(ResultSet rs,Object object) throws JPMapperException;
	/**
	 * 
	 * @param object
	 * @throws JPMapperException
	 */
	public abstract Object map(Object object) throws JPMapperException;
	/**
	 * 
	 * @param object
	 * @param where
	 * @return
	 * @throws JPMapperException
	 */
	public abstract Object map(Object object,String where) throws JPMapperException;
	
	/**
	 * 
	 * @param object
	 * @param step
	 * @param start
	 * @param where
	 * @param order
	 * @throws JPMapperException
	 */
	public abstract ArrayList mapAll(Object object, String where, long step, long start, String order) throws JPMapperException;
	
	public ArrayList mapAll(Object object) throws JPMapperException{
		return mapAll(object,null,-1,-1,null);
	}
	public ArrayList mapAll(Object object,String where) throws JPMapperException{
		return mapAll(object,where,-1,-1,null);
	}
	
	/**
	 * 
	 * @throws JPMapperException
	 */
	public abstract void rollback();

	/**
	 * 
	 * @param object
	 * @param where
	 * @throws JPMapperException
	 */
	public abstract int update(Object object, String where) throws JPMapperException;

	
	public DBInformation getDbInformation() {
		return dbInformation;
	}

	public void setDbInformation(DBInformation dbInformation) {
		this.dbInformation = dbInformation;
	}

	public HashMap<String, Class> getDbType2JavaType() {
		return dbType2JavaType;
	}

	public HashMap<String, String> getDbTypeWithDelimiter() {
		return this.dbTypeWithDelimiter;
	}

	public HashMap<Class, String> getJavaTypeWithDelimiter() {
		return this.javaTypeWithDelimiter;
	}

	public Connection getConnection() {
		return connection;
	}

	
	
	
	

	
	
}