package jpmapper.core;

/**
 * @author Dahhane walid , mindware.ma
 * @version 0.7.1
 */

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.sql.Connection;
import java.sql.Date;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Properties;
import java.util.Set;
import java.util.Map.Entry;

import jpmapper.config.DBInformation;
import jpmapper.exceptions.JPMapperException;
import jpmapper.exceptions.LockWaitTimeoutException;

public class JPMapperFacade {

	private static JPMapperFacade instance;
	protected DBInformation dbInformation;
	private JPMapperException ormException;
	protected DBMapperFactory dbmsMapperFactory;
	private boolean isTransactionalMode;
	protected AbstractDBMapper dbmsMapper;
	protected String configFilePath;

	protected JPMapperFacade() {
	}

	@SuppressWarnings("unchecked")
	protected JPMapperFacade(String configFilePath, boolean isTransactionalMode) {
		this.configFilePath = configFilePath;
		String errorMessage = "Error in FacadeORM constructor";
		Properties properties = new Properties();
		try {
			properties.load(new FileInputStream(configFilePath));
		} catch (FileNotFoundException e) {
			this.ormException = new JPMapperException();
			this.ormException.setErrorMessage(errorMessage);
			this.ormException.setOrmErrorMessage(e.getMessage());
			this.ormException.setStackTrace(e.getStackTrace());
			e.printStackTrace();
		} catch (IOException e) {
			this.ormException = new JPMapperException();
			this.ormException.setErrorMessage(errorMessage);
			this.ormException.setOrmErrorMessage(e.getMessage());
			this.ormException.setStackTrace(e.getStackTrace());
			e.printStackTrace();
		}
		dbInformation = new DBInformation();
		dbInformation.setDbName(properties.getProperty("dbName"));
		dbInformation.setHost(properties.getProperty("host"));
		dbInformation.setDbUser(properties.getProperty("dbUser"));
		dbInformation.setDbPassword(properties.getProperty("dbPassword"));
		dbInformation.setDbms(properties.getProperty("dbms"));
		String port = properties.getProperty("port");
		dbInformation.setDbFieldDelimiter(properties
				.getProperty("dbFieldDelimiter"));
		int portNumber = 0;
		try {
			portNumber = Integer.parseInt(port);
			dbInformation.setPort(portNumber);
		} catch (Exception e) {
			this.ormException = new JPMapperException();
			this.ormException.setErrorMessage(errorMessage);
			this.ormException.setOrmErrorMessage(e.getMessage());
			this.ormException.setStackTrace(e.getStackTrace());
			e.printStackTrace();
		}
		HashMap<String, Class> javaTypeAsString2Class = new HashMap<String, Class>();
		javaTypeAsString2Class.put("String", String.class);
		javaTypeAsString2Class.put("int", int.class);
		javaTypeAsString2Class.put("long", long.class);
		javaTypeAsString2Class.put("double", double.class);
		javaTypeAsString2Class.put("float", float.class);
		javaTypeAsString2Class.put("Date", Date.class);
		javaTypeAsString2Class.put("Timestamp", Timestamp.class);
		javaTypeAsString2Class.put("Integer", int.class);
		javaTypeAsString2Class.put("Float", float.class);
		javaTypeAsString2Class.put("Double", double.class);
		javaTypeAsString2Class.put("Long", long.class);

		HashMap<String, Class> dbType2JavaType = new HashMap<String, Class>();
		HashMap<String, String> dbTypeWithQuote = new HashMap<String, String>();
		HashMap<Class, String> javaTypeWithQuote = new HashMap<Class, String>();
		;

		Set<Entry<Object, Object>> data = properties.entrySet();
		Iterator iterator = data.iterator();
		while (iterator.hasNext()) {
			Entry<String, String> oneData = (Entry<String, String>) iterator
					.next();
			String key = oneData.getKey();
			if (key.startsWith("db2java")) {
				dbType2JavaType.put(key.substring(key.lastIndexOf(".") + 1),
						javaTypeAsString2Class.get(oneData.getValue()));
			}
			if (key.startsWith("dbFieldValueDelimiter")) {
				dbTypeWithQuote.put(key.substring(key.lastIndexOf(".") + 1),
						oneData.getValue());
			}
			if (key.startsWith("javaFieldValueDelimiter")) {
				javaTypeWithQuote.put(javaTypeAsString2Class.get(key
						.substring(key.lastIndexOf(".") + 1)), oneData
						.getValue());
			}
		}

		dbInformation.setDbType2JavaType(dbType2JavaType);
		dbInformation.setDbTypeWithDelimiter(dbTypeWithQuote);
		dbInformation.setJavaTypeWithDelimiter(javaTypeWithQuote);
		this.dbmsMapperFactory = DBMapperFactory.getInstance(dbInformation,
				!isTransactionalMode);
		this.dbmsMapper = this.dbmsMapperFactory.getDBMSMapper(dbInformation
				.getDbms());
		this.isTransactionalMode = isTransactionalMode;
		if (this.isTransactionalMode) {
			try {
				this.getDbmsMapper().connect(this.getDbInformation().getHost(),
						this.getDbInformation().getPort(),
						this.getDbInformation().getDbName(),
						this.getDbInformation().getDbUser(),
						this.getDbInformation().getDbPassword(), false);
			} catch (JPMapperException e) {
				e.printStackTrace();
			}
		}
	}

	public Transaction getNewTransaction() {
		return new Transaction(configFilePath);
	}
	public static JPMapperFacade getDBInstance(String dbmsName,boolean isTransactionMode){
		String configFilePath = null;
		if(dbmsName.equalsIgnoreCase("mysql")){
			configFilePath="dbMySQLinfo.properties";
		}
		if (instance == null)
			instance = new JPMapperFacade(configFilePath, isTransactionMode);

		instance.isTransactionalMode = isTransactionMode;
		return instance;
	}
	public static JPMapperFacade getInstance(String configFilePath,
			boolean isTransactionMode) {
		if (instance != null && isTransactionMode == true) {// connect
			try {
				instance.getDbmsMapper().connect(
						instance.getDbInformation().getHost(),
						instance.getDbInformation().getPort(),
						instance.getDbInformation().getDbName(),
						instance.getDbInformation().getDbUser(),
						instance.getDbInformation().getDbPassword(), false);
			} catch (JPMapperException e) {
				e.printStackTrace();
			}
		}
		if (instance == null)
			instance = new JPMapperFacade(configFilePath, isTransactionMode);

		instance.isTransactionalMode = isTransactionMode;
		return instance;
	}

	public static JPMapperFacade getInstance(String configFilePath) {
		if (instance == null)
			instance = new JPMapperFacade(configFilePath, false);
		instance.isTransactionalMode = false;
		return instance;
	}

	public static JPMapperFacade getInstance() {
		if (instance == null)
			instance = new JPMapperFacade();
		return instance;
	}

	public JPMapperException getOrmException() {
		return this.ormException;
	}

	public void setOrmException(JPMapperException ormException) {
		this.ormException = ormException;
	}

	public DBInformation getDbInformation() {
		return dbInformation;
	}

	public void setDbInformation(DBInformation dbInformation) {
		this.dbInformation = dbInformation;
	}

	public boolean isTransactionalMode() {
		return isTransactionalMode;
	}

	/*
	 * public void setTransactionalMode(boolean isTransactionalMode) {
	 * this.isTransactionalMode = isTransactionalMode; }
	 */

	public DBMapperFactory getDbmsMapperFactory() {
		return dbmsMapperFactory;
	}

	public AbstractDBMapper getDbmsMapper() {
		return dbmsMapper;
	}

	public Object get(Object object) {
		if (this.isTransactionalMode()) {
			try {
				return this.getDbmsMapper().map(object);
			} catch (JPMapperException e) {
				e.printStackTrace();
				System.out.println(e.getOrmErrorMessage());
				return null;
			}
		}
		try {
			this.getDbmsMapper().connect(this.getDbInformation().getHost(),
					this.getDbInformation().getPort(),
					this.getDbInformation().getDbName(),
					this.getDbInformation().getDbUser(),
					this.getDbInformation().getDbPassword(), true);
			Object objectNew = this.getDbmsMapper().map(object);
			this.getDbmsMapper().disconnect();
			return objectNew;
		} catch (JPMapperException e) {
			this.getDbmsMapper().disconnect();
			System.out.println(e.getOrmErrorMessage());
			e.printStackTrace();
			return null;
		}
	}
	
	public Object get(Object object,String where) {
		if (this.isTransactionalMode()) {
			try {
				return this.getDbmsMapper().map(object,where);
			} catch (JPMapperException e) {
				e.printStackTrace();
				System.out.println(e.getOrmErrorMessage());
				return null;
			}
		}
		try {
			this.getDbmsMapper().connect(this.getDbInformation().getHost(),
					this.getDbInformation().getPort(),
					this.getDbInformation().getDbName(),
					this.getDbInformation().getDbUser(),
					this.getDbInformation().getDbPassword(), true);
			Object objectNew = this.getDbmsMapper().map(object,where);
			this.getDbmsMapper().disconnect();
			return objectNew;
		} catch (JPMapperException e) {
			this.getDbmsMapper().disconnect();
			System.out.println(e.getOrmErrorMessage());
			e.printStackTrace();
			return null;
		}
	}


	
	
	
	public Object get(Object parentObject,Object object){
		return this.getDbmsMapper().map(parentObject, object);
	}
	public ArrayList getAll(Object parentObject,Object arrayList){
		try {
			return this.getDbmsMapper().mapAll(parentObject, arrayList);
		} catch (IllegalArgumentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SecurityException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (JPMapperException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}
	public ArrayList getAll(Object object, String where, long start, long step,
			String order) {
		if (this.isTransactionalMode()) {
			try {
				return this.getDbmsMapper().mapAll(object, where, step, start,
						order);
			} catch (JPMapperException e) {
				e.printStackTrace();
				System.out.println(e.getOrmErrorMessage());
				return null;
			}
		}
		try {
			this.getDbmsMapper().connect(this.getDbInformation().getHost(),
					this.getDbInformation().getPort(),
					this.getDbInformation().getDbName(),
					this.getDbInformation().getDbUser(),
					this.getDbInformation().getDbPassword(), true);
			ArrayList list = this.getDbmsMapper().mapAll(object, where, step,
					start, order);
			this.getDbmsMapper().disconnect();
			return list;
		} catch (JPMapperException e) {
			this.getDbmsMapper().disconnect();
			e.printStackTrace();
			System.out.println(e.getOrmErrorMessage());
			return null;
		}
	}


	public ArrayList getAll(Object object){
		return getAll(object,null,-1,-1,null);
	}
	public ArrayList getAll(Object object,String where){
		return getAll(object,where,-1,-1,null);
	}
	public boolean put(Object object){
		boolean inserted = false;
		if (this.isTransactionalMode()) {
			try {
				return this.getDbmsMapper().insert(object);
			} catch (JPMapperException e) {
				e.printStackTrace();
				System.out.println(e.getOrmErrorMessage());
				return inserted;
			}
		}
		try {
			this.getDbmsMapper().connect(this.getDbInformation().getHost(),
					this.getDbInformation().getPort(),
					this.getDbInformation().getDbName(),
					this.getDbInformation().getDbUser(),
					this.getDbInformation().getDbPassword(), true);
			inserted = this.getDbmsMapper().insert(object);
			this.getDbmsMapper().disconnect();
		} catch (JPMapperException e) {
			e.printStackTrace();
			System.out.println(e.getOrmErrorMessage());
			this.getDbmsMapper().rollback();
			this.getDbmsMapper().disconnect();
			return inserted;
		}
		return inserted;
	}
	
	public ArrayList buildObjectList(ResultSet rs,Object object){
		if(this.isTransactionalMode()){
			try {
				return this.getDbmsMapper().mapAll(rs,object);
			} catch (JPMapperException e) {
				e.printStackTrace();
				return null;
			}
		}
		try {
			this.getDbmsMapper().connect(this.getDbInformation().getHost(), this.getDbInformation().getPort(), this.getDbInformation().getDbName(), this.getDbInformation().getDbUser(), this.getDbInformation().getDbPassword(),true);
			ArrayList list=this.getDbmsMapper().mapAll(rs,object);
			this.getDbmsMapper().disconnect();
			return list;
		} catch (JPMapperException e) {
			this.getDbmsMapper().disconnect();
			e.printStackTrace();
			return null;
		}
	}
	
	public Object buildObject(ResultSet rs,Object object){
		if(this.isTransactionalMode()){
			try {
				return this.getDbmsMapper().map(rs,object);
			} catch (JPMapperException e) {
				e.printStackTrace();
				return null;
			}
		}
		try {
			this.getDbmsMapper().connect(this.getDbInformation().getHost(), this.getDbInformation().getPort(), this.getDbInformation().getDbName(), this.getDbInformation().getDbUser(), this.getDbInformation().getDbPassword(),true);
			Object obj=this.getDbmsMapper().map(rs,object);
			this.getDbmsMapper().disconnect();
			return obj;
		} catch (JPMapperException e) {
			this.getDbmsMapper().disconnect();
			e.printStackTrace();
			return null;
		}
	}

	
	public int update(Object object){
		return update(object,null);
	}
	public int delete(Object object){
		try {
			return delete(object,null);
		} catch (LockWaitTimeoutException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return -1;
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return -1;
		}
	}
	public int update(Object object, String where) {
		if (this.isTransactionalMode()) {
			try {
				return this.getDbmsMapper().update(object, where);
			} catch (JPMapperException e) {
				e.printStackTrace();
				System.out.println(e.getOrmErrorMessage());
				return -1;
			}
		}
		try {
			this.getDbmsMapper().connect(this.getDbInformation().getHost(),
					this.getDbInformation().getPort(),
					this.getDbInformation().getDbName(),
					this.getDbInformation().getDbUser(),
					this.getDbInformation().getDbPassword(), true);
			int updated = this.getDbmsMapper().update(object, where);
			this.getDbmsMapper().disconnect();
			return updated;
		} catch (JPMapperException e) {
			e.printStackTrace();
			System.out.println(e.getOrmErrorMessage());
			this.getDbmsMapper().rollback();
			this.getDbmsMapper().disconnect();
			return -1;
		}
	}

	public int delete(Object object, String where) throws LockWaitTimeoutException, SQLException  {
		if (this.isTransactionalMode()) {
			try {
				return this.getDbmsMapper().delete(object, where);
			} catch (Exception e) {
				e.printStackTrace();
				System.out.println(e.getMessage());
				return -1;
			}
		}
		try {
			this.getDbmsMapper().connect(this.getDbInformation().getHost(),
					this.getDbInformation().getPort(),
					this.getDbInformation().getDbName(),
					this.getDbInformation().getDbUser(),
					this.getDbInformation().getDbPassword(), true);
			int deleted = this.getDbmsMapper().delete(object, where);
			this.getDbmsMapper().disconnect();
			return deleted;
		} catch (JPMapperException e) {
			e.printStackTrace();
			System.out.println(e.getOrmErrorMessage());
			this.getDbmsMapper().rollback();
			this.getDbmsMapper().disconnect();
			return -1;
		} catch (LockWaitTimeoutException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return 0;
	}

	public boolean execute(String sqlQuery) {
		boolean returnValue = false;
		if (this.isTransactionalMode()) {
			try {
				return this.getDbmsMapper().execute(sqlQuery);
			} catch (JPMapperException e) {
				e.printStackTrace();
				System.out.println(e.getOrmErrorMessage());
				return returnValue;
			}
		}
		try {
			this.getDbmsMapper().connect(this.getDbInformation().getHost(),
					this.getDbInformation().getPort(),
					this.getDbInformation().getDbName(),
					this.getDbInformation().getDbUser(),
					this.getDbInformation().getDbPassword(), true);
			returnValue = this.getDbmsMapper().execute(sqlQuery);
			this.getDbmsMapper().disconnect();
			return returnValue;
		} catch (JPMapperException e) {
			e.printStackTrace();
			System.out.println(e.getOrmErrorMessage());
			this.getDbmsMapper().rollback();
			this.getDbmsMapper().disconnect();
			return returnValue;
		}
	}

	public int executeUpdate(String sqlQuery) {
		int returnValue = -1;
		if (this.isTransactionalMode()) {
			try {
				return this.getDbmsMapper().executeUpdate(sqlQuery);
			} catch (JPMapperException e) {
				e.printStackTrace();
				System.out.println(e.getOrmErrorMessage());
				return returnValue;
			}
		}
		try {
			this.getDbmsMapper().connect(this.getDbInformation().getHost(),
					this.getDbInformation().getPort(),
					this.getDbInformation().getDbName(),
					this.getDbInformation().getDbUser(),
					this.getDbInformation().getDbPassword(), true);
			returnValue = this.getDbmsMapper().executeUpdate(sqlQuery);
			this.getDbmsMapper().disconnect();
			return returnValue;
		} catch (JPMapperException e) {
			e.printStackTrace();
			System.out.println(e.getOrmErrorMessage());
			this.getDbmsMapper().rollback();
			this.getDbmsMapper().disconnect();
			return returnValue;
		}
	}

	/**
	 * <p>
	 * This method should be run in transactional mode
	 * </p>
	 * 
	 * @param sqlQuery
	 * @return ResultSet
	 * 
	 */
	public ResultSet executeQuery(String sqlQuery) {
		if (this.isTransactionalMode()) {
			try {
				return this.getDbmsMapper().executeQuery(sqlQuery);
			} catch (JPMapperException e) {
				e.printStackTrace();
				System.out.println(e.getOrmErrorMessage());
				return null;
			}
		}
		try {
			this.getDbmsMapper().connect(this.getDbInformation().getHost(),
					this.getDbInformation().getPort(),
					this.getDbInformation().getDbName(),
					this.getDbInformation().getDbUser(),
					this.getDbInformation().getDbPassword(), true);
			ResultSet rset = this.getDbmsMapper().executeQuery(sqlQuery);
			this.getDbmsMapper().disconnect();
			return rset;
		} catch (JPMapperException e) {
			e.printStackTrace();
			System.out.println(e.getOrmErrorMessage());
			this.getDbmsMapper().rollback();
			this.getDbmsMapper().disconnect();
			return null;
		}
	}

	public long getNextAutoIncrement(Object object) {
		if (this.isTransactionalMode()) {
			try {
				return this.getDbmsMapper().getNextAutoIncrement(object);
			} catch (JPMapperException e) {
				e.printStackTrace();
				System.out.println(e.getOrmErrorMessage());
				return -1;
			}
		}
		try {
			this.getDbmsMapper().connect(this.getDbInformation().getHost(),
					this.getDbInformation().getPort(),
					this.getDbInformation().getDbName(),
					this.getDbInformation().getDbUser(),
					this.getDbInformation().getDbPassword(), true);
			long nextValue = this.getDbmsMapper().getNextAutoIncrement(object);
			this.getDbmsMapper().disconnect();
			return nextValue;
		} catch (JPMapperException e) {
			e.printStackTrace();
			System.out.println(e.getOrmErrorMessage());
			this.getDbmsMapper().rollback();
			this.getDbmsMapper().disconnect();
			return -1;
		}
	}

	public Connection getConnection() {
		if (this.isTransactionalMode()) {

			try {
				if (this.getDbmsMapper().getConnection() != null
						&& !this.getDbmsMapper().getConnection().isClosed())
					;
				else
					this.getDbmsMapper().connect(
							this.getDbInformation().getHost(),
							this.getDbInformation().getPort(),
							this.getDbInformation().getDbName(),
							this.getDbInformation().getDbUser(),
							this.getDbInformation().getDbPassword(), false);

				return this.getDbmsMapper().getConnection();
			} catch (JPMapperException e) {
				e.printStackTrace();
				System.out.println(e.getOrmErrorMessage());
				return null;
			} catch (SQLException e) {
				e.printStackTrace();
				return null;
			}
		}

		try {
			this.getDbmsMapper().connect(this.getDbInformation().getHost(),
					this.getDbInformation().getPort(),
					this.getDbInformation().getDbName(),
					this.getDbInformation().getDbUser(),
					this.getDbInformation().getDbPassword(), true);
			return this.getDbmsMapper().getConnection();
		} catch (JPMapperException e) {
			e.printStackTrace();
			System.out.println(e.getOrmErrorMessage());
			return null;
		}
	}

}
