package jpmapper.database;

/**
 * @author Dahhane walid 
 * @version 0.7.1
 */

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import jpmapper.config.DBInformation;
import jpmapper.core.AbstractDBMapper;
import jpmapper.core.FOREIGN;
import jpmapper.core.ID;
import jpmapper.exceptions.JPMapperException;
import jpmapper.exceptions.LockWaitTimeoutException;
import com.mysql.jdbc.jdbc2.optional.MysqlDataSource;

public class MySqlMapper extends AbstractDBMapper{
	
	private static MySqlMapper instance;	
	
	private MySqlMapper(DBInformation dbInformation) {
		this.dbType2JavaType=dbInformation.getDbType2JavaType();
		this.dbTypeWithDelimiter=dbInformation.getDbTypeWithDelimiter();
		this.javaTypeWithDelimiter=dbInformation.getJavaTypeWithDelimiter();
		this.dbInformation=dbInformation;
		
	}
	
	public static MySqlMapper getInstance(DBInformation dbInformation,boolean isSingleton){
		if(!isSingleton)return new MySqlMapper(dbInformation);
		if(instance==null)instance=new MySqlMapper(dbInformation);
		return instance;
		}
	
	@Override
	public void connect(String host,int port, String dbName,String user, String password,boolean isAutoCommit) throws JPMapperException {
		String errorMessage="Error in"+this.getClass().getSimpleName()+":: int connect(String host,int port, String dbName,String user, String password) method";
		try {
			if(isAutoCommit&&this.connection!=null && !this.connection.isClosed()){
				return;
			}
		}catch (SQLException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		}
		MysqlDataSource ods=new MysqlDataSource();
		ods.setDatabaseName(dbName+"?useUnicode=true&characterEncoding=utf8");
		ods.setPortNumber(port);
		ods.setServerName(host);
		ods.setUser(user);
		ods.setPassword(password);
		try {
			this.connection=ods.getConnection();
			this.connection.setAutoCommit(isAutoCommit);
		} catch (SQLException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		}
	}
	
	
	
	@Override
	public boolean execute(String sqlQuery) throws JPMapperException{
		
		boolean returnValue=true;
		String errorMessage="Error in"+this.getClass().getSimpleName()+":: int execute(String sqlQuery) method";
		try {
			Statement stmt = this.connection.createStatement();
			stmt.execute(sqlQuery);
			return returnValue;
		} catch (SQLException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		}
	}
	
	@Override
	public int executeUpdate(String sqlQuery) throws JPMapperException{
		String errorMessage="Error in"+this.getClass().getSimpleName()+":: int executeUpdate(String sqlQuery) method";
		try {
			Statement stmt = this.connection.createStatement();
			return stmt.executeUpdate(sqlQuery);
		} catch (SQLException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		}
	}

	@Override
	public ResultSet executeQuery(String sqlQuery) throws JPMapperException{
		
		String errorMessage="Error in"+this.getClass().getSimpleName()+":: int execute(String sqlQuery) method";
		
		try {
			Statement stmt = this.connection.createStatement();
			ResultSet rset=null;
			rset=stmt.executeQuery(sqlQuery);
			return rset;
		} catch (SQLException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		}
	}
	
	@Override
	public ArrayList mapAll(ResultSet rs,Object object) throws JPMapperException{
		String errorMessage="Error in"+this.getClass().getSimpleName()+":: ArrayList map(ResultSet rs,Object object) method";
		Class classObject = object.getClass();
		ArrayList list=new ArrayList();
		try{
			ResultSetMetaData metaData = rs.getMetaData();
			/*****************OBJECT MAPPING****************************/
			while(rs.next()){
				object=object.getClass().newInstance();
				int nbrColumns=metaData.getColumnCount();
				//Get all columns, it means that the object must have all fields existing on the Table
				for (int i = 1; i <=nbrColumns ; i++) {
					try {
						classObject.getMethod("set" + (metaData.getColumnLabel(i).charAt(0)+"").toUpperCase()+metaData.getColumnLabel(i).substring(1),this.dbType2JavaType.get(metaData.getColumnTypeName(i)) )
						.invoke(object, rs.getClass().getMethod("get"+(this.dbType2JavaType.get(metaData.getColumnTypeName(i)).getSimpleName().charAt(0)+"").toUpperCase()+this.dbType2JavaType.get(metaData.getColumnTypeName(i)).getSimpleName().substring(1),int.class).invoke(rs, i));
						
					} catch (NoSuchMethodException e) {
						e.printStackTrace();
					}catch (InvocationTargetException e) {
						try {
							Object time=null;
							if(metaData.getColumnTypeName(i).equalsIgnoreCase("TIMESTAMP")||metaData.getColumnTypeName(i).equalsIgnoreCase("DATE")||metaData.getColumnTypeName(i).equalsIgnoreCase("DATETIME")){
								
									classObject.getMethod("set" + (metaData.getColumnLabel(i).charAt(0)+"").toUpperCase()+metaData.getColumnLabel(i).substring(1),this.dbType2JavaType.get(metaData.getColumnTypeName(i)) )
									.invoke(object, time);
							}								
							
						} catch (SQLException e1) {
							e1.printStackTrace();
						} catch (NoSuchMethodException e1) {
							e1.printStackTrace();
						}catch (IllegalArgumentException e1) {
							e1.printStackTrace();
						} catch (SecurityException e1) {
							e1.printStackTrace();
						} catch (IllegalAccessException e1) {
							e1.printStackTrace();
						} catch (InvocationTargetException e1) {
							e1.printStackTrace();
						}
					}
				}//end for
				list.add(object);
			}//end while
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (SQLException e) {
			e.printStackTrace();
		}catch (IllegalAccessException e) {
			e.printStackTrace();
		}
			return list;
	}
	
	@Override
	public Object map(ResultSet rs,Object object) throws JPMapperException{
		String errorMessage="Error in"+this.getClass().getSimpleName()+":: Object map(ResultSet rs,Object object) method";
		Class classObject = object.getClass();
		try{
			ResultSetMetaData metaData = rs.getMetaData();
			/*****************OBJECT MAPPING****************************/
			while(rs.next()){
				object=object.getClass().newInstance();
				int nbrColumns=metaData.getColumnCount();
				//Get all columns, it means that the object must have all fields existing on the Table
				for (int i = 1; i <=nbrColumns ; i++) {
					try {
						classObject.getMethod("set" + (metaData.getColumnLabel(i).charAt(0)+"").toUpperCase()+metaData.getColumnLabel(i).substring(1),this.dbType2JavaType.get(metaData.getColumnTypeName(i)) )
						.invoke(object, rs.getClass().getMethod("get"+(this.dbType2JavaType.get(metaData.getColumnTypeName(i)).getSimpleName().charAt(0)+"").toUpperCase()+this.dbType2JavaType.get(metaData.getColumnTypeName(i)).getSimpleName().substring(1),int.class).invoke(rs, i));
						
					} catch (NoSuchMethodException e) {
						e.printStackTrace();
					}catch (InvocationTargetException e) {
						try {
							Object time=null;
							if(metaData.getColumnTypeName(i).equalsIgnoreCase("TIMESTAMP")||metaData.getColumnTypeName(i).equalsIgnoreCase("DATE")||metaData.getColumnTypeName(i).equalsIgnoreCase("DATETIME")){
								
									classObject.getMethod("set" + (metaData.getColumnLabel(i).charAt(0)+"").toUpperCase()+metaData.getColumnLabel(i).substring(1),this.dbType2JavaType.get(metaData.getColumnTypeName(i)) )
									.invoke(object, time);
							}								
							
						} catch (SQLException e1) {
							e1.printStackTrace();
						} catch (NoSuchMethodException e1) {
							e1.printStackTrace();
						}catch (IllegalArgumentException e1) {
							e1.printStackTrace();
						} catch (SecurityException e1) {
							e1.printStackTrace();
						} catch (IllegalAccessException e1) {
							e1.printStackTrace();
						} catch (InvocationTargetException e1) {
							e1.printStackTrace();
						}
					}
				}//end for
				
			}//end while
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (SQLException e) {
			e.printStackTrace();
		}catch (IllegalAccessException e) {
			e.printStackTrace();
		}
		return object;
	}

	
	public ArrayList mapAll(Object parentObject,Object arrayList) throws JPMapperException, IllegalArgumentException, SecurityException, IllegalAccessException, InvocationTargetException, NoSuchMethodException{
		
		String ID = null;
		//get object id
		for(int i=0;i<parentObject.getClass().getDeclaredFields().length;i++)
			if(parentObject.getClass().getDeclaredFields()[i].isAnnotationPresent(ID.class)){
				ID=parentObject.getClass().getDeclaredFields()[i].getName();
				break;
				}

		return mapAll(arrayList,ID+"="+parentObject.getClass().getMethod("get"+(ID.charAt(0)+"").toUpperCase()+ID.substring(1),null).invoke(parentObject, null));
		
	}
	
	public Object map(Object parentObject,Object object){
		
		//Get the object id from it's parent
		
		//Get object id
		String tableName = null;
		String ID = null;
		String foreignID = null;
		
		//get object id
		for(int i=0;i<parentObject.getClass().getDeclaredFields().length;i++)
			if(parentObject.getClass().getDeclaredFields()[i].isAnnotationPresent(ID.class)){
				ID=parentObject.getClass().getDeclaredFields()[i].getName();
				break;
				}
		
		//get object foreignID
		for(int i=0;i<object.getClass().getDeclaredFields().length;i++)
			if(object.getClass().getDeclaredFields()[i].isAnnotationPresent(ID.class)){
				foreignID=object.getClass().getDeclaredFields()[i].getName();
				break;
				}
		
		Field dbTableField=null;
		try {
			dbTableField = parentObject.getClass().getDeclaredField("_name");
			tableName=(String) dbTableField.get(parentObject);
			if(tableName!=null);
			else tableName=parentObject.getClass().getSimpleName();
		} catch (NoSuchFieldException e) {
			tableName=parentObject.getClass().getSimpleName();
		} catch (IllegalArgumentException e) {
		} catch (IllegalAccessException e) {
		}
		//Prepare request
		tableName=this.dbInformation.getDbFieldDelimiter()+tableName+this.dbInformation.getDbFieldDelimiter();		

		String query = null;
		try {
			String methodName="get"+(ID.charAt(0)+"").toUpperCase()+ID.substring(1);
			query = "SELECT "+this.dbInformation.getDbFieldDelimiter()+foreignID+this.dbInformation.getDbFieldDelimiter()+" FROM "+tableName+" WHERE "+this.dbInformation.getDbFieldDelimiter()+ID+this.dbInformation.getDbFieldDelimiter()+"="+
			this.javaTypeWithDelimiter.get(parentObject.getClass().getMethod(methodName,null).getGenericReturnType())+parentObject.getClass().getMethod(methodName,null).invoke(parentObject, null)+this.javaTypeWithDelimiter.get(parentObject.getClass().getMethod(methodName,null).getGenericReturnType());
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		Statement stmt;
		ResultSet result;
		try {
			stmt = this.connection.createStatement();
			result = stmt.executeQuery(query.toString());
		if(!result.next())return object;
		try {
			//attention dbdilmiter!!!!!!!!!!!
			int keyPosition = 1;
			for(int i=1;i<=result.getMetaData().getColumnCount();i++)
				if(result.getMetaData().getColumnName(i).equals(foreignID))
					keyPosition=i;
			
			object.getClass().getMethod("set"+(foreignID.charAt(0)+"").toUpperCase()+foreignID.substring(1), dbType2JavaType.get(result.getMetaData().getColumnTypeName(keyPosition))).invoke(object,result.getObject(foreignID));
			
			return object=map(object);
		} catch (JPMapperException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SecurityException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	
		
		return object;
		
	}
	
	@Override
	public Object map(Object object) throws JPMapperException{		
		
		String errorMessage="Error in"+this.getClass().getSimpleName().toLowerCase()+":: Object map(Object object) method";
		
		/***********************PREPARE THE QUERY***************************/
		//get object id
		String ID = null;
		for(int i=0;i<object.getClass().getDeclaredFields().length;i++)
			if(object.getClass().getDeclaredFields()[i].isAnnotationPresent(ID.class)){
				ID=object.getClass().getDeclaredFields()[i].getName();
				break;
				}
		
		StringBuffer query=new StringBuffer("SELECT * FROM ");
		String dbTableName=null;;
		Class<? extends Object> classObject = object.getClass();
		Field dbTableField=null;
		try {
			dbTableField = classObject.getDeclaredField("_name");
			dbTableName=(String) dbTableField.get(object);
			if(dbTableName!=null);
			else dbTableName=object.getClass().getSimpleName();
		} catch (NoSuchFieldException e) {
			dbTableName=object.getClass().getSimpleName();
		} catch (IllegalArgumentException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		} catch (IllegalAccessException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		}

		query.append(this.dbInformation.getDbFieldDelimiter()+dbTableName+this.dbInformation.getDbFieldDelimiter());		
		String methodName="get"+(ID.charAt(0)+"").toUpperCase()+ID.substring(1);
		try {
			query.append(" WHERE "+this.dbInformation.getDbFieldDelimiter()+ID+this.dbInformation.getDbFieldDelimiter()+"="+this.javaTypeWithDelimiter.get(object.getClass().getMethod(methodName,null).getGenericReturnType())+object.getClass().getMethod(methodName,null).invoke(object, null)+this.javaTypeWithDelimiter.get(object.getClass().getMethod(methodName,null).getGenericReturnType()));
		} catch (NoSuchMethodException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		} catch (SecurityException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		} catch (IllegalArgumentException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		} catch (IllegalAccessException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		} catch (InvocationTargetException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		}
			
		/***********************END OF PREPARE THE QUERY***************************/				
		
		ResultSet result=null;
		ResultSetMetaData metaData=null;
		int nbrColumns=0;
		try{
			Statement stmt = this.connection.createStatement();
			result=stmt.executeQuery(query.toString());
			if(!result.next())return object;
			metaData= result.getMetaData();
			nbrColumns=metaData.getColumnCount();
		}catch (SQLException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		}
		
		/*****************OBJECT MAPPING****************************/
		
		//Get all columns, it means that the object must have all fields existing on the Table
		for (int i = 1; i <=nbrColumns ; i++) {
			try {
				classObject.getMethod("set" + (metaData.getColumnLabel(i).charAt(0)+"").toUpperCase()+metaData.getColumnLabel(i).substring(1),this.dbType2JavaType.get(metaData.getColumnTypeName(i)) )
				.invoke(object, result.getClass().getMethod("get"+(this.dbType2JavaType.get(metaData.getColumnTypeName(i)).getSimpleName().charAt(0)+"").toUpperCase()+this.dbType2JavaType.get(metaData.getColumnTypeName(i)).getSimpleName().substring(1),int.class).invoke(result, i));
				
			} catch (NoSuchMethodException e) {
//// chagement de la conception du problème
				System.out.println("Warning: No such methode exception "+e.getMessage());
				continue;

				
				//throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
			}catch (InvocationTargetException e) {
				try {
					Object time=null;
					if(metaData.getColumnTypeName(i).equalsIgnoreCase("TIMESTAMP")||metaData.getColumnTypeName(i).equalsIgnoreCase("DATE")||metaData.getColumnTypeName(i).equalsIgnoreCase("DATETIME")){
						
							classObject.getMethod("set" + (metaData.getColumnLabel(i).charAt(0)+"").toUpperCase()+metaData.getColumnLabel(i).substring(1),this.dbType2JavaType.get(metaData.getColumnTypeName(i)) )
							.invoke(object, time);
					}								
					
				} catch (SQLException e1) {
					throw new JPMapperException(0,errorMessage,e1.getMessage(),e1.getStackTrace());
				} catch (NoSuchMethodException e2) {
					throw new JPMapperException(0,errorMessage,e2.getMessage(),e2.getStackTrace());
				}catch (IllegalArgumentException e3) {
					throw new JPMapperException(0,errorMessage,e3.getMessage(),e3.getStackTrace());
				} catch (SecurityException e4) {
					throw new JPMapperException(0,errorMessage,e4.getMessage(),e4.getStackTrace());
				} catch (IllegalAccessException e5) {
					throw new JPMapperException(0,errorMessage,e5.getMessage(),e5.getStackTrace());
				} catch (InvocationTargetException e6) {
					throw new JPMapperException(0,errorMessage,e6.getMessage(),e6.getStackTrace());
				}
			}catch (SecurityException e) {
				throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
			} catch (IllegalAccessException e) {
				throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
			}catch (IllegalArgumentException e) {
				throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
			} catch (SQLException e) {
				throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
			} 
			
			/*****************END OF OBJECT MAPPING****************************/
		}
		return object;
	}

	@Override
	public Object map(Object object,String where) throws JPMapperException{		
		
		String errorMessage="Error in"+this.getClass().getSimpleName().toLowerCase()+":: Object map(Object object) method";
		
		/***********************PREPARE THE QUERY***************************/
		//get object id
		String ID = null;
		for(int i=0;i<object.getClass().getDeclaredFields().length;i++)
			if(object.getClass().getDeclaredFields()[i].isAnnotationPresent(ID.class)){
				ID=object.getClass().getDeclaredFields()[i].getName();
				break;
				}
		
		StringBuffer query=new StringBuffer("SELECT * FROM ");
		String dbTableName=null;;
		Class<? extends Object> classObject = object.getClass();
		Field dbTableField=null;
		try {
			dbTableField = classObject.getDeclaredField("_name");
			dbTableName=(String) dbTableField.get(object);
			if(dbTableName!=null);
			else dbTableName=object.getClass().getSimpleName();
		} catch (NoSuchFieldException e) {
			dbTableName=object.getClass().getSimpleName();
		} catch (IllegalArgumentException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		} catch (IllegalAccessException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		}

		query.append(this.dbInformation.getDbFieldDelimiter()+dbTableName+this.dbInformation.getDbFieldDelimiter());		
		String methodName="get"+(ID.charAt(0)+"").toUpperCase()+ID.substring(1);
		try {
			query.append(" WHERE "+where);
		} catch (SecurityException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		} catch (IllegalArgumentException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		}
			
		/***********************END OF PREPARE THE QUERY***************************/				
		
		ResultSet result=null;
		ResultSetMetaData metaData=null;
		int nbrColumns=0;
		try{
			Statement stmt = this.connection.createStatement();
			result=stmt.executeQuery(query.toString());
			if(!result.next())return object;
			metaData= result.getMetaData();
			nbrColumns=metaData.getColumnCount();
		}catch (SQLException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		}
		
		/*****************OBJECT MAPPING****************************/
		
		//Get all columns, it means that the object must have all fields existing on the Table
		for (int i = 1; i <=nbrColumns ; i++) {
			try {
				classObject.getMethod("set" + (metaData.getColumnLabel(i).charAt(0)+"").toUpperCase()+metaData.getColumnLabel(i).substring(1),this.dbType2JavaType.get(metaData.getColumnTypeName(i)) )
				.invoke(object, result.getClass().getMethod("get"+(this.dbType2JavaType.get(metaData.getColumnTypeName(i)).getSimpleName().charAt(0)+"").toUpperCase()+this.dbType2JavaType.get(metaData.getColumnTypeName(i)).getSimpleName().substring(1),int.class).invoke(result, i));
				
			} catch (NoSuchMethodException e) {
//// chagement de la conception du problème
				System.out.println("Warning: No such methode exception "+e.getMessage());
				continue;

				
				//throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
			}catch (InvocationTargetException e) {
				try {
					Object time=null;
					if(metaData.getColumnTypeName(i).equalsIgnoreCase("TIMESTAMP")||metaData.getColumnTypeName(i).equalsIgnoreCase("DATE")||metaData.getColumnTypeName(i).equalsIgnoreCase("DATETIME")){
						
							classObject.getMethod("set" + (metaData.getColumnLabel(i).charAt(0)+"").toUpperCase()+metaData.getColumnLabel(i).substring(1),this.dbType2JavaType.get(metaData.getColumnTypeName(i)) )
							.invoke(object, time);
					}								
					
				} catch (SQLException e1) {
					throw new JPMapperException(0,errorMessage,e1.getMessage(),e1.getStackTrace());
				} catch (NoSuchMethodException e2) {
					throw new JPMapperException(0,errorMessage,e2.getMessage(),e2.getStackTrace());
				}catch (IllegalArgumentException e3) {
					throw new JPMapperException(0,errorMessage,e3.getMessage(),e3.getStackTrace());
				} catch (SecurityException e4) {
					throw new JPMapperException(0,errorMessage,e4.getMessage(),e4.getStackTrace());
				} catch (IllegalAccessException e5) {
					throw new JPMapperException(0,errorMessage,e5.getMessage(),e5.getStackTrace());
				} catch (InvocationTargetException e6) {
					throw new JPMapperException(0,errorMessage,e6.getMessage(),e6.getStackTrace());
				}
			}catch (SecurityException e) {
				throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
			} catch (IllegalAccessException e) {
				throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
			}catch (IllegalArgumentException e) {
				throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
			} catch (SQLException e) {
				throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
			} 
			
			/*****************END OF OBJECT MAPPING****************************/
		}
		return object;
	}

	
	
	
	//insert object into database
	@Override
	public boolean insert(Object object) throws JPMapperException{
		//this is the error message wrapped into jpmapperException (!will be deleted)
		String errorMessage="Error in"+this.getClass().getSimpleName()+":: int insert(Object object) method";
		boolean error=false;
		//table Name
		String dbTableName=null;
		Class<? extends Object> classObject = object.getClass();
		Field dbTableField=null;
		try {
			//get the table name from the attribute _name if existe
			dbTableField = classObject.getDeclaredField("_name");
			dbTableName=(String) dbTableField.get(object);
			if(dbTableName!=null);
			else dbTableName=object.getClass().getSimpleName();
		} catch (Exception e) {
			//if not set the table name to object class's name
			dbTableName=object.getClass().getSimpleName();
		}
		//Prepare the statement
		
		StringBuffer fields=new StringBuffer("("),values=new StringBuffer(" VALUES("),query=new StringBuffer("INSERT INTO");
		{String ID = null;
		for(int i=0;i<object.getClass().getDeclaredFields().length;i++)
			if(object.getClass().getDeclaredFields()[i].isAnnotationPresent(ID.class)){
				ID=object.getClass().getDeclaredFields()[i].getName();
				Class IDType=object.getClass().getDeclaredFields()[i].getType();
				try {
					try {
						//Get Object ID and set it to autoIncrement if is null
						Object objValue=object.getClass().getMethod("get"+(ID.charAt(0)+"").toUpperCase()+ID.substring(1), null).invoke(object,null);
						Long longValue=Long.parseLong(String.valueOf(objValue));
						if(longValue.longValue()==0){
							long nextAutoIncrementValue=this.getNextAutoIncrement(object);
							if(IDType==int.class){
								int nextValueInt=(int)nextAutoIncrementValue;
								object.getClass().getMethod("set"+(ID.charAt(0)+"").toUpperCase()+ID.substring(1), IDType).invoke(object,nextValueInt);
							}else//it is a long
								object.getClass().getMethod("set"+(ID.charAt(0)+"").toUpperCase()+ID.substring(1), IDType).invoke(object,nextAutoIncrementValue);
						}
					}catch (NumberFormatException e) {//the object ID is neither an int nor long
						// TODO: do nothing
					}catch (Exception e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				} catch (SecurityException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				
				break;
				}}
		
		
		ResultSet result=null;
		ResultSetMetaData metaData = null;
		try {
			result = this.connection.createStatement().executeQuery("SELECT * FROM "+this.dbInformation.getDbFieldDelimiter()+dbTableName+this.dbInformation.getDbFieldDelimiter()+" where 1!=1");
			metaData=result.getMetaData();
			for(int i=1;i<=metaData.getColumnCount();i++){
				String methodName="get" + (metaData.getColumnLabel(i).charAt(0)+"").toUpperCase()+metaData.getColumnLabel(i).substring(1);
				try {
					if(object.getClass().getMethod(methodName,null).invoke(object, null)!=null){//field is set thus it should be updated
						
						try{
						
							values.append(dbTypeWithDelimiter.get(metaData.getColumnTypeName(i))+object.getClass().getMethod("get" + (metaData.getColumnLabel(i).charAt(0)+"").toUpperCase()+metaData.getColumnLabel(i).substring(1),null).invoke(object,null));
							values.append(dbTypeWithDelimiter.get(metaData.getColumnTypeName(i))+",");
						}catch(Exception e){
							String delimiter="null";
							
							if((delimiter=dbTypeWithDelimiter.get(metaData.getColumnTypeName(i))+" "+dbTypeWithDelimiter.get(metaData.getColumnTypeName(i)+",")).equals(","))
								values.append(delimiter);
							else
								values.append("null,");				
						}
							fields.append(this.dbInformation.getDbFieldDelimiter()+metaData.getColumnLabel(i)+this.dbInformation.getDbFieldDelimiter()+",");	
					}
				} catch (Exception e){
					continue;
				}
				}
			//Get foreigns key values
			{
			for(int i=0;i<object.getClass().getDeclaredFields().length;i++) {
				if(object.getClass().getDeclaredFields()[i].isAnnotationPresent(FOREIGN.class)&&!object.getClass().getDeclaredFields()[i].getType().getSimpleName().toLowerCase().equals("arraylist")){
					try {
						try {
							//Get Object ID and set it to autoIncrement if is null
							Object Foreign=object.getClass().getMethod("get"+(object.getClass().getDeclaredFields()[i].getName().charAt(0)+"").toUpperCase()+(object.getClass().getDeclaredFields()[i].getName()).substring(1),null).invoke(object, null);
							for(int j=0;j<Foreign.getClass().getDeclaredFields().length;j++){
								if(Foreign.getClass().getDeclaredFields()[j].isAnnotationPresent(ID.class)){
									String FOREIGN=Foreign.getClass().getDeclaredFields()[j].getName();
									if(Foreign.getClass().getMethod("get" + (FOREIGN.charAt(0)+"").toUpperCase()+
											FOREIGN.substring(1),null).invoke(Foreign,null)==null || Foreign.getClass().getMethod("get" + (FOREIGN.charAt(0)+"").toUpperCase()+
													FOREIGN.substring(1),null).invoke(Foreign,null).toString().equals("0"))continue;
								fields.append(this.dbInformation.getDbFieldDelimiter()+FOREIGN+this.dbInformation.getDbFieldDelimiter()+",");
								values.append(javaTypeWithDelimiter.get(Foreign.getClass().getDeclaredFields()[j].getType())+
										Foreign.getClass().getMethod("get" + (FOREIGN.charAt(0)+"").toUpperCase()+
												FOREIGN.substring(1),null).invoke(Foreign,null));
								values.append(javaTypeWithDelimiter.get(Foreign.getClass().getDeclaredFields()[j].getType())+",");
									
								}
								}
									} catch (Exception e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
					} catch (SecurityException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					
					//break;
					}
			}}
			
			values.replace(values.length()-1, values.length(), "");
			fields.replace(fields.length()-1, fields.length(), "");
			query.append(" "+this.dbInformation.getDbFieldDelimiter()+dbTableName+this.dbInformation.getDbFieldDelimiter());
			query.append(" "+fields+")");
			query.append(" "+values+")");
			Statement stmt = this.connection.createStatement();
			stmt.executeUpdate(query.toString());
			return true;
			
		} catch (SQLException e) {
				e.printStackTrace();
				throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		} catch (IllegalArgumentException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		} catch (SecurityException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		}		
	}
	
	//delete object from database
	@Override
	public int delete(Object object,String where) throws LockWaitTimeoutException, SQLException{
		String errorMessage="Error in"+this.getClass().getSimpleName()+":: int delete(Object object,String where) method";
		int deleted=0;
		int error=-1;
		String dbTableName = object.getClass().getSimpleName();
		Class<? extends Object> classObject = object.getClass();
		Field dbTableField=null;
		try {
			dbTableField = classObject.getDeclaredField("_name");
			dbTableName=(String) dbTableField.get(object);
		} catch (Exception E){
		}
		
		String ID = null;
		
		for(int i=0;i<object.getClass().getDeclaredFields().length;i++)
			if(object.getClass().getDeclaredFields()[i].isAnnotationPresent(ID.class)){
				ID=object.getClass().getDeclaredFields()[i].getName();
				break;
			}
		
		String query="DELETE FROM "+this.dbInformation.getDbFieldDelimiter()+dbTableName+this.dbInformation.getDbFieldDelimiter();
		if(where!=null){
			query+=" WHERE "+where;
		}else{
			try {
				query+=" WHERE "+this.dbInformation.getDbFieldDelimiter()+ID+this.dbInformation.getDbFieldDelimiter()+"="+this.javaTypeWithDelimiter.get(object.getClass().getMethod("get"+(ID.charAt(0)+"").toUpperCase()+ID.substring(1), null).getGenericReturnType())+object.getClass().getMethod("get"+(ID.charAt(0)+"").toUpperCase()+ID.substring(1), null).invoke(object, null)+this.javaTypeWithDelimiter.get(object.getClass().getMethod("get"+(ID.charAt(0)+"").toUpperCase()+ID.substring(1), null).getGenericReturnType());
			} catch (Exception e) {
				
			}
		}
		
		
		try{
			Statement stmt=this.connection.createStatement();
			deleted=stmt.executeUpdate(query);
			return deleted;
			
			}catch(SQLException e){
			if(e.getMessage().equals("Lock wait timeout exceeded; try restarting transaction"))	
			throw new LockWaitTimeoutException(e.getMessage());
			
			throw new SQLException();
			}
	}

	//update object
	public int update(Object object,String where) throws JPMapperException{
		String errorMessage="Error in"+this.getClass().getSimpleName()+":: int update(Object object,String where) method";
		int error=-1;
		String dbTableName;
		Class<? extends Object> classObject = object.getClass();
		Field dbTableField=null;
		try {
			dbTableField = classObject.getDeclaredField("_name");
			dbTableName=(String) dbTableField.get(object);
			if(dbTableName!=null);
			else dbTableName=object.getClass().getSimpleName();
		} catch (NoSuchFieldException e1) {
			dbTableName=object.getClass().getSimpleName();
			//e1.printStackTrace();
		}catch (IllegalArgumentException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		} catch (IllegalAccessException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		}
		String fk="";
		//get identifier
		String ID = null;
		String value;
		for(int i=0;i<object.getClass().getDeclaredFields().length;i++){
			if(object.getClass().getDeclaredFields()[i].isAnnotationPresent(ID.class)){
				ID=object.getClass().getDeclaredFields()[i].getName();
			}
			//Get foreign object ids and their values, if foreign objects are set
			if(object.getClass().getDeclaredFields()[i].isAnnotationPresent(FOREIGN.class)&&!object.getClass().getDeclaredFields()[i].getType().getSimpleName().toLowerCase().equals("arraylist")){
				try {
					try {
						//Get Object ID
						Object Foreign=object.getClass().getMethod("get"+(object.getClass().getDeclaredFields()[i].getName().charAt(0)+"").toUpperCase()+(object.getClass().getDeclaredFields()[i].getName()).substring(1),null).invoke(object, null);
						if(Foreign!=null){//foreign object is set
							for(int j=0;j<Foreign.getClass().getDeclaredFields().length;j++){
								if(Foreign.getClass().getDeclaredFields()[j].isAnnotationPresent(ID.class)){
									String FOREIGN=Foreign.getClass().getDeclaredFields()[j].getName();
									fk+=this.dbInformation.getDbFieldDelimiter()+FOREIGN+this.dbInformation.getDbFieldDelimiter()+
									"="+javaTypeWithDelimiter.get(Foreign.getClass().getDeclaredFields()[j].getType())+Foreign.getClass().getMethod("get" + (FOREIGN.charAt(0)+"").toUpperCase()+FOREIGN.substring(1),null).invoke(Foreign,null)+
									javaTypeWithDelimiter.get(Foreign.getClass().getDeclaredFields()[j].getType());
									fk+=",";
								}
							}
						}
						} catch (Exception e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
				} catch (SecurityException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
		StringBuffer query=new StringBuffer("UPDATE "+this.dbInformation.getDbFieldDelimiter()+dbTableName+this.dbInformation.getDbFieldDelimiter()+" SET");
		ResultSet result=null;
		ResultSetMetaData metaData = null;
		try {
			result = this.connection.createStatement().executeQuery("SELECT * FROM "+this.dbInformation.getDbFieldDelimiter()+dbTableName+this.dbInformation.getDbFieldDelimiter()+" where 1!=1");
			metaData=result.getMetaData();
			int i=1;
			for(i=1;i<=metaData.getColumnCount();i++){
				try{
				String methodName="get" + (metaData.getColumnLabel(i).charAt(0)+"").toUpperCase()+metaData.getColumnLabel(i).substring(1);
				if(object.getClass().getMethod(methodName,null).invoke(object, null)!=null){//field is set thus it should be updated
					String field=metaData.getColumnLabel(i);
					query.append(" "+this.dbInformation.getDbFieldDelimiter()+field+this.dbInformation.getDbFieldDelimiter()+"="+dbTypeWithDelimiter.get(metaData.getColumnTypeName(i))+object.getClass().getMethod(methodName,null).invoke(object, null)+dbTypeWithDelimiter.get(metaData.getColumnTypeName(i))+",");	
				}
				}catch(Exception e){
					System.out.println(e.getMessage());
				}
			}
			query.append(fk);
				query.replace(query.length()-1, query.length(), "");//remove last "," caracter
		/*Method[] methods=object.getClass().getDeclaredMethods();
		try {
			for(int i=0;i<methods.length;i++)
				if(methods[i].getName().startsWith("get")){
					if(methods[i].invoke(object, null)!=null && !methods[i].getName().equals("get"+(ID.charAt(0)+"").toUpperCase()+ID.substring(1))){//field is set thus it should be updated, && field is different than object id
						String field=methods[i].getName().substring(3);
						field=(field.charAt(0)+"").toLowerCase()+field.substring(1);
						query.append(" "+this.dbInformation.getDbFieldDelimiter()+field+this.dbInformation.getDbFieldDelimiter()+"="+javaTypeWithDelimiter.get(methods[i].getGenericReturnType())+methods[i].invoke(object, null)+javaTypeWithDelimiter.get(methods[i].getGenericReturnType())+",");
					}
				}*/
			
			
			
			if(where!=null)
				query.append(" WHERE "+where);
			else{
				String methodName="get"+(ID.charAt(0)+"").toUpperCase()+ID.substring(1);
				query.append(" WHERE "+this.dbInformation.getDbFieldDelimiter()+ID+this.dbInformation.getDbFieldDelimiter()+"="+javaTypeWithDelimiter.get(object.getClass().getMethod(methodName,null).getGenericReturnType())+object.getClass().getMethod(methodName,null).invoke(object, null)+javaTypeWithDelimiter.get(object.getClass().getMethod(methodName,null).getGenericReturnType()));
			}
			
		}catch (SQLException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		}catch (IllegalArgumentException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		} catch (IllegalAccessException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		} catch (InvocationTargetException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		}catch (SecurityException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		} catch (NoSuchMethodException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		}
		
		try{
			Statement stmt=this.connection.createStatement();
			int updated=stmt.executeUpdate(query.toString());
			return updated;
		}catch (SQLException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		}
	}
	
	@Override
	public void commit(){
		String errorMessage="Error in"+this.getClass().getSimpleName()+":: int commit() method";
		try {
			this.connection.commit();
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	@Override
	public void rollback(){
		String errorMessage="Error in"+this.getClass().getSimpleName()+":: int rollback() method";
		try {
			this.connection.rollback();
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	@Override
	public void disconnect(){
		String errorMessage="Error in"+this.getClass().getSimpleName()+":: int disconnect() method";
		try{
			if(this.connection!=null && !this.connection.isClosed())
			this.connection.close();
		}catch(SQLException e){
			e.printStackTrace();
		}
		
	}

	@Override
	public long getNextAutoIncrement(Object object) throws JPMapperException{
		String errorMessage="Error in"+this.getClass().getSimpleName()+":: int getNextAutoIncrementValue(Object object) method";
		int error=-1;
		String dbTableName=null;
		Class<? extends Object> classObject = object.getClass();
		Field dbTableField=null;
		try {
			dbTableField = classObject.getDeclaredField("_name");
			dbTableName=(String) dbTableField.get(object); 
			if(dbTableName!=null);
			else dbTableName=object.getClass().getSimpleName();
		}catch (NoSuchFieldException e1) {
			dbTableName=object.getClass().getSimpleName();
			//e1.printStackTrace();
		}catch (IllegalArgumentException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		} catch (IllegalAccessException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		}
		String query="SHOW TABLE STATUS LIKE '"+dbTableName+"'";
				
		long nextValue=-1;
		try{
			Statement stmt=this.connection.createStatement();
			ResultSet rset=null;
			rset=stmt.executeQuery(query);
			while(rset.next()){
				nextValue=rset.getLong("Auto_increment");				
			}
			return  nextValue;
			}catch(SQLException e){
				throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		}
	}

	

	@Override
	public ArrayList mapAll(Object object,String where, long step, long start ,String order) throws JPMapperException{
		String errorMessage="Error in"+this.getClass().getSimpleName().toLowerCase()+":: mapAll(Object object, long step, long start, String where,String order)";
		
		/***********************PREPARE THE QUERY***************************/
		//get object id
		String ID = null;
		for(int i=0;i<object.getClass().getDeclaredFields().length;i++)
			if(object.getClass().getDeclaredFields()[i].isAnnotationPresent(ID.class)){
				ID=object.getClass().getDeclaredFields()[i].getName();
				break;
				}
		
		StringBuffer query=new StringBuffer("SELECT * FROM ");
		String dbTableName=null;;
		Class<? extends Object> classObject = object.getClass();
		Field dbTableField=null;
		try {
			dbTableField = classObject.getDeclaredField("_name");
			dbTableName=(String) dbTableField.get(object);
			if(dbTableName!=null);
			else dbTableName=object.getClass().getSimpleName();
		} catch (NoSuchFieldException e) {
			dbTableName=object.getClass().getSimpleName();
		} catch (IllegalArgumentException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		} catch (IllegalAccessException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		}
		query.append(this.dbInformation.getDbFieldDelimiter()+dbTableName+this.dbInformation.getDbFieldDelimiter());		
		String methodName="get"+(ID.charAt(0)+"").toUpperCase()+ID.substring(1);
		try {
			if(where==null)
				;//query.append(" WHERE "+this.dbInformation.getDbFieldDelimiter()+ID+this.dbInformation.getDbFieldDelimiter()+"="+this.javaTypeWithDelimiter.get(object.getClass().getMethod(methodName,null).getGenericReturnType())+object.getClass().getMethod(methodName,null).invoke(object, null)+this.javaTypeWithDelimiter.get(object.getClass().getMethod(methodName,null).getGenericReturnType()));
			else
				query.append(" WHERE "+where);
			if(order!=null)
				query.append(" ORDER BY "+order);
			if(start>=0 && step>=0)
				query.append(" LIMIT "+start+","+step);
				
		} catch (SecurityException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		} catch (IllegalArgumentException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		}
			
		/***********************END OF PREPARE THE QUERY***************************/

		ArrayList list=new ArrayList();
		ResultSet result=null;
		ResultSetMetaData metaData=null;
		int nbrColumns=0;
		try{
			Statement stmt = this.connection.createStatement();
			result=stmt.executeQuery(query.toString());
			//if(!result.next())return object;
			
			/*****************OBJECT MAPPING****************************/
			while(result.next()){
				object=object.getClass().newInstance();
				metaData= result.getMetaData();
				nbrColumns=metaData.getColumnCount();
				//Get all columns, it means that the object must have all fields existing on the Table
				for (int i = 1; i <=nbrColumns ; i++) {
					try {
						classObject.getMethod("set" + (metaData.getColumnLabel(i).charAt(0)+"").toUpperCase()+metaData.getColumnLabel(i).substring(1),this.dbType2JavaType.get(metaData.getColumnTypeName(i)) )
						.invoke(object, result.getClass().getMethod("get"+(this.dbType2JavaType.get(metaData.getColumnTypeName(i)).getSimpleName().charAt(0)+"").toUpperCase()+this.dbType2JavaType.get(metaData.getColumnTypeName(i)).getSimpleName().substring(1),int.class).invoke(result, i));
						
					} catch (NoSuchMethodException e) {
						continue;
						//throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
					}catch (InvocationTargetException e) {
						try {
							Object time=null;
							if(metaData.getColumnTypeName(i).equalsIgnoreCase("TIMESTAMP")||metaData.getColumnTypeName(i).equalsIgnoreCase("DATE")||metaData.getColumnTypeName(i).equalsIgnoreCase("DATETIME")){
								
									classObject.getMethod("set" + (metaData.getColumnLabel(i).charAt(0)+"").toUpperCase()+metaData.getColumnLabel(i).substring(1),this.dbType2JavaType.get(metaData.getColumnTypeName(i)) )
									.invoke(object, time);
							}								
							
						} catch (SQLException e1) {
							throw new JPMapperException(0,errorMessage,e1.getMessage(),e1.getStackTrace());
						} catch (NoSuchMethodException e2) {
							throw new JPMapperException(0,errorMessage,e2.getMessage(),e2.getStackTrace());
						}catch (IllegalArgumentException e3) {
							throw new JPMapperException(0,errorMessage,e3.getMessage(),e3.getStackTrace());
						} catch (SecurityException e4) {
							throw new JPMapperException(0,errorMessage,e4.getMessage(),e4.getStackTrace());
						} catch (IllegalAccessException e5) {
							throw new JPMapperException(0,errorMessage,e5.getMessage(),e5.getStackTrace());
						} catch (InvocationTargetException e6) {
							throw new JPMapperException(0,errorMessage,e6.getMessage(),e6.getStackTrace());
						}
					}
				}//end for
				list.add(object);
			}//end while
		}catch (SQLException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		}catch (SecurityException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		} catch (IllegalAccessException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		}catch (IllegalArgumentException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		}catch (InstantiationException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		}
			
			/*****************END OF OBJECT MAPPING****************************/
		return list;
		}
	
	
	@Override
	public ArrayList mapAll(ObjectData objectData,String where, long step, long start ,String order) throws JPMapperException{
		String errorMessage="Error in"+this.getClass().getSimpleName()+":: mapAll(ObjectData objectData, long step, long start, String where,String order)";
		Object object=objectData.getObject();
		/***********************PREPARE THE QUERY***************************/
		//get object id
		String ID = null;
		for(int i=0;i<object.getClass().getDeclaredFields().length;i++)
			if(object.getClass().getDeclaredFields()[i].isAnnotationPresent(ID.class)){
				ID=object.getClass().getDeclaredFields()[i].getName();
				break;
				}
		
		StringBuffer query=new StringBuffer("SELECT * FROM ");
		String dbTableName=null;;
		Class<? extends Object> classObject = object.getClass();
		Field dbTableField=null;
		try {
			dbTableField = classObject.getDeclaredField("_name");
			dbTableName=(String) dbTableField.get(object);
			if(dbTableName!=null);
			else dbTableName=object.getClass().getSimpleName();
		} catch (NoSuchFieldException e) {
			dbTableName=object.getClass().getSimpleName();
		} catch (IllegalArgumentException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		} catch (IllegalAccessException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		}

		query.append(this.dbInformation.getDbFieldDelimiter()+dbTableName+this.dbInformation.getDbFieldDelimiter());		
		String methodName="get"+(ID.charAt(0)+"").toUpperCase()+ID.substring(1);
		try {
			if(where==null)
				query.append(" WHERE "+this.dbInformation.getDbFieldDelimiter()+ID+this.dbInformation.getDbFieldDelimiter()+"="+this.javaTypeWithDelimiter.get(object.getClass().getMethod(methodName,null).getGenericReturnType())+object.getClass().getMethod(methodName,null).invoke(object, null)+this.javaTypeWithDelimiter.get(object.getClass().getMethod(methodName,null).getGenericReturnType()));
			else
				query.append(" WHERE "+where);
			if(order!=null)
				query.append(" ORDER BY "+order);
			if(start>=0 && step>=0)
				query.append(" LIMIT "+start+","+step);
				
		} catch (NoSuchMethodException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		} catch (SecurityException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		} catch (IllegalArgumentException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		} catch (IllegalAccessException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		} catch (InvocationTargetException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		}
			
		/***********************END OF PREPARE THE QUERY***************************/

		ArrayList list=new ArrayList();
		ResultSet result=null;
		ResultSetMetaData metaData=null;
		int nbrColumns=0;
		try{
			Statement stmt = this.connection.createStatement();
			result=stmt.executeQuery(query.toString());
			//if(!result.next())return object;
			
			/*****************OBJECT MAPPING****************************/
			while(result.next()){
				object=object.getClass().newInstance();
				metaData= result.getMetaData();
				nbrColumns=metaData.getColumnCount();
				//Get all columns, it means that the object must have all fields existing on the Table
				for (int i = 1; i <=nbrColumns ; i++) {
					try {
						classObject.getMethod("set" + (metaData.getColumnLabel(i).charAt(0)+"").toUpperCase()+metaData.getColumnLabel(i).substring(1),this.dbType2JavaType.get(metaData.getColumnTypeName(i)) )
						.invoke(object, result.getClass().getMethod("get"+(this.dbType2JavaType.get(metaData.getColumnTypeName(i)).getSimpleName().charAt(0)+"").toUpperCase()+this.dbType2JavaType.get(metaData.getColumnTypeName(i)).getSimpleName().substring(1),int.class).invoke(result, i));
						
					} catch (NoSuchMethodException e) {
						throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
					}catch (InvocationTargetException e) {
						try {
							Object time=null;
							if(metaData.getColumnTypeName(i).equalsIgnoreCase("TIMESTAMP")||metaData.getColumnTypeName(i).equalsIgnoreCase("DATE")||metaData.getColumnTypeName(i).equalsIgnoreCase("DATETIME")){
								
									classObject.getMethod("set" + (metaData.getColumnLabel(i).charAt(0)+"").toUpperCase()+metaData.getColumnLabel(i).substring(1),this.dbType2JavaType.get(metaData.getColumnTypeName(i)) )
									.invoke(object, time);
							}								
							
						} catch (SQLException e1) {
							throw new JPMapperException(0,errorMessage,e1.getMessage(),e1.getStackTrace());
						} catch (NoSuchMethodException e2) {
							throw new JPMapperException(0,errorMessage,e2.getMessage(),e2.getStackTrace());
						}catch (IllegalArgumentException e3) {
							throw new JPMapperException(0,errorMessage,e3.getMessage(),e3.getStackTrace());
						} catch (SecurityException e4) {
							throw new JPMapperException(0,errorMessage,e4.getMessage(),e4.getStackTrace());
						} catch (IllegalAccessException e5) {
							throw new JPMapperException(0,errorMessage,e5.getMessage(),e5.getStackTrace());
						} catch (InvocationTargetException e6) {
							throw new JPMapperException(0,errorMessage,e6.getMessage(),e6.getStackTrace());
						}
					}
				}//end for
				objectData.setObject(object);
				object=this.map(objectData, true);
				list.add(object);
			}//end while
		}catch (SQLException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		}catch (SecurityException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		} catch (IllegalAccessException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		}catch (IllegalArgumentException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		}catch (InstantiationException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		}
			
			/*****************END OF OBJECT MAPPING****************************/
		return list; 
	}
	
	
	
	public Object walkThroughTree(Object parentObject,ObjectData od) throws JPMapperException{
		Object object=od.getObject();
		String errorMessage="Error in"+this.getClass().getSimpleName()+":: Object walkThroughTree(parentObject parentObject,ObjectData objectData) method";
		
		/***********************PREPARE THE QUERY***************************/
		//get object id
		String ID = null;
		for(int i=0;i<parentObject.getClass().getDeclaredFields().length;i++)
			if(parentObject.getClass().getDeclaredFields()[i].isAnnotationPresent(ID.class)){
				ID=parentObject.getClass().getDeclaredFields()[i].getName();
				break;
				}
		
		StringBuffer query=new StringBuffer("SELECT * FROM ");
		String dbTableName=null;
		Class<? extends Object> classObject = object.getClass();
		Field dbTableField=null;
		try {
			dbTableField = classObject.getDeclaredField("_name");
			dbTableName=(String) dbTableField.get(object);
			if(dbTableName!=null);
			else dbTableName=object.getClass().getSimpleName();
		} catch (NoSuchFieldException e) {
			dbTableName=object.getClass().getSimpleName();
		} catch (IllegalArgumentException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		} catch (IllegalAccessException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		}

		query.append(this.dbInformation.getDbFieldDelimiter()+dbTableName+this.dbInformation.getDbFieldDelimiter());		
		String methodName="get"+(ID.charAt(0)+"").toUpperCase()+ID.substring(1);
		try {
			String where = this.dbInformation.getDbFieldDelimiter()+ID+this.dbInformation.getDbFieldDelimiter()+"="+this.javaTypeWithDelimiter.get(parentObject.getClass().getMethod(methodName,null).getGenericReturnType())+parentObject.getClass().getMethod(methodName,null).invoke(parentObject, null)+this.javaTypeWithDelimiter.get(parentObject.getClass().getMethod(methodName,null).getGenericReturnType());
			ArrayList subObjectList=this.mapAll(object, where, -1, -1, null);
			
			try{//check if object has list of subObjects
				parentObject.getClass().getMethod("set" + object.getClass().getSimpleName()+"List",ArrayList.class)
				.invoke(parentObject, subObjectList);
				for(int k=0;k<subObjectList.size();k++){//get subobjects for each object in the list
					object=subObjectList.get(k);
					if(od.getObjectDataList()!=null && od.getObjectDataList().size()>0){
						int j=0;
						while(j<od.getObjectDataList().size()){
							this.walkThroughTree(object,od.getObjectDataList().get(j));
							j++;
						}
					}
					subObjectList.set(k,object);
				}
			}catch (NoSuchMethodException e) {//object has not list of subObjects in its attribute list (fields)
				if(subObjectList.size()==0)//no subject found
					parentObject.getClass().getMethod("set" + object.getClass().getSimpleName(),object.getClass())
					.invoke(parentObject, object.getClass().newInstance());
				if(subObjectList.size()==1){//
					parentObject.getClass().getMethod("set" + object.getClass().getSimpleName(),object.getClass())
					.invoke(parentObject, subObjectList.get(0));
					object=subObjectList.get(0);
					od.setObject(object);
					if(od.getObjectDataList()!=null && od.getObjectDataList().size()>0){
						int j=0;
						while(j<od.getObjectDataList().size()){
							this.walkThroughTree(object,od.getObjectDataList().get(j));
							j++;
						}
					}
				}
			}
		} catch (InstantiationException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		} catch (NoSuchMethodException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		} catch (SecurityException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		} catch (IllegalArgumentException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		} catch (IllegalAccessException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		} catch (InvocationTargetException e) {
			throw new JPMapperException(0,errorMessage,e.getMessage(),e.getStackTrace());
		}
			
		/***********************END OF PREPARE THE QUERY***************************/
		
		return parentObject;
		
		}
	

}

